@@ -23,7 +23,12 @@ from pandas import DataFrame       # type: ignore
 import sys
 import traceback
 from datetime import datetime
-from typing import Final, List, Dict, Set, Any, Type, Optional
+if sys.version_info >= (3, 8):
+    from typing import Final, List, Dict, Set, Any, Type, Optional
+else:
+    # backwards compatibility with Python 35, Python36, and Python37 
+    from typing import List, Dict, Set, Any, Type, Optional
+    from typing_extensions import Final
 
 
 class CancelledException(Exception):
@@ -71,12 +76,12 @@ class ResultSet(object):
     the specific values being stored alongside each result. There
     may be multiple results for the same parameters, to allow for
     repetition of experiments at a data point.
+    Results committ5ed to result sets are immutable: once entered, a result can't be deleted
+    or changed.
+
     Result sets also record "pending" results, allowing us to record experiments
     in progress. A pending result can be finalised by providing it with a
-    value, or can be deleted.
-
-    Result sets are immutable: once entered, a result can't be deleted
-    or changed. Pending results can however be cancelled.
+    value, or can be cancelled.
 
     A result set can be used very Pythonically using a :term:`results dict` holding
     the metadata, parameters, and results of experiments. For larger experiment
@@ -619,6 +624,7 @@ class ResultSet(object):
         if dtype.kind in self.ZeroMapping:
             return self.ZeroMapping[dtype.kind]
         else:
+            print('No zero value for type {dt}, using 0.0'.format(dt=dtype), file=sys.stderr)
             return 0.0     # and hope for the best
 
 
